"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _ResolvedMessage_str;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResolvedMessage = void 0;
const message_value_1 = require("./message-value");
const MESSAGE = 'message';
function getPattern(context, message) {
    switch (message.type) {
        case 'message':
            return { pattern: message.pattern.body };
        case 'select': {
            const ctx = message.selectors.map(sel => ({
                selector: context.resolve(sel),
                best: null,
                keys: null
            }));
            let candidates = message.variants;
            loop: for (let i = 0; i < ctx.length; ++i) {
                const sc = ctx[i];
                if (!sc.keys) {
                    sc.keys = new Set();
                    for (const { keys } of candidates) {
                        const key = keys[i];
                        if (!key)
                            break loop; // key-mismatch error
                        if (key.type !== '*')
                            sc.keys.add(key.value);
                    }
                }
                sc.best = sc.keys.size ? sc.selector.selectKey(sc.keys) : null;
                // Leave out all candidate variants that aren't the best,
                // or only the catchall ones, if nothing else matches.
                candidates = candidates.filter(v => {
                    const k = v.keys[i];
                    if (k.type === '*')
                        return sc.best == null;
                    return sc.best === k.value;
                });
                // If we've run out of candidates,
                // drop the previous best key of the preceding selector,
                // reset all subsequent key sets,
                // and restart the loop.
                if (candidates.length === 0) {
                    if (i === 0)
                        break; // No match; should not happen
                    const prev = ctx[i - 1];
                    if (prev.best == null)
                        prev.keys?.clear();
                    else
                        prev.keys?.delete(prev.best);
                    for (let j = i; j < ctx.length; ++j)
                        ctx[j].keys = null;
                    candidates = message.variants;
                    i = -1;
                }
            }
            const res = candidates[0];
            return res
                ? { pattern: res.value.body }
                : { pattern: [], meta: { selectResult: 'no-match' } };
        }
        default:
            return { pattern: [{ type: 'junk', source: message.source }] };
    }
}
/**
 * The result of resolving a {@link MessageFormat} message.
 *
 * @beta
 */
class ResolvedMessage extends message_value_1.MessageValue {
    constructor(context, message, source) {
        const { meta, pattern } = getPattern(context, message);
        const resMsg = pattern.map(elem => context.resolve(elem));
        super(MESSAGE, context, resMsg, { meta, source });
        // Cache for string value
        _ResolvedMessage_str.set(this, void 0);
    }
    selectKey(keys) {
        let hasError = false;
        const str = this.toString(() => (hasError = true));
        return !hasError && keys.has(str) ? str : null;
    }
    toString(onError) {
        if (typeof __classPrivateFieldGet(this, _ResolvedMessage_str, "f") !== 'string') {
            __classPrivateFieldSet(this, _ResolvedMessage_str, '', "f");
            for (const mv of this.value)
                __classPrivateFieldSet(this, _ResolvedMessage_str, __classPrivateFieldGet(this, _ResolvedMessage_str, "f") + mv.toString(onError), "f");
        }
        return __classPrivateFieldGet(this, _ResolvedMessage_str, "f");
    }
}
exports.ResolvedMessage = ResolvedMessage;
_ResolvedMessage_str = new WeakMap();
